@@ -3,7 +3,7 @@
  * config.h
  *	  load/save configuration
  *
- * Portions Copyright (c) 2017-2019 Pavel Stehule
+ * Portions Copyright (c) 2017-2020 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/config.h
@@ -19,6 +19,7 @@
 typedef struct
 {
 	char   *pathname;
+	char   *log_pathname;
 	bool	ignore_case;
 	bool	ignore_lower_case;
 	bool	no_sound;
@@ -44,6 +45,7 @@ typedef struct
 	char	csv_separator;
 	char	csv_header;			/* a - auto, - off, + on */
 	char   *nullstr;
+	char   *csv_skip_columns_like;
 	bool	ignore_short_rows;
 	char	double_header;
 	int		border_type;
@@ -57,6 +59,8 @@ typedef struct
 	bool	force_password_prompt;
 	char   *password;
 	char   *dbname;
+	bool	watch_file;
+	bool	quit_on_f3;
 } Options;
 
 extern bool save_config(char *path, Options *opts);
