@@ -3,7 +3,7 @@
  * menu.c
  *	 holds menu related code
  *
- * Portions Copyright (c) 2017-2019 Pavel Stehule
+ * Portions Copyright (c) 2017-2020 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/menu.c
@@ -21,19 +21,28 @@
 #define MENU_ITEM_OPTIONS			2
 
 ST_CMDBAR_ITEM _bottombar[] = {
-	{"Save", false, 2, cmd_SaveData},
-	{"Search", false, 7, cmd_ForwardSearch},
-	{"Menu", false, 9, cmd_ShowMenu},
-	{"Quit", false, 10, cmd_Quit},
-	{NULL}
+	{"Save", false, 2, cmd_SaveData, 0},
+	{"Search", false, 7, cmd_ForwardSearch, 0},
+	{"Menu", false, 9, cmd_ShowMenu, 0},
+	{"Quit", false, 10, cmd_Quit, 0},
+	{NULL, false, 0, 0, 0}
+};
+
+ST_CMDBAR_ITEM _bottombar_alt1[] = {
+	{"Save", false, 2, cmd_SaveData, 0},
+	{"Quit", false, 3, cmd_Quit, 0},
+	{"Search", false, 7, cmd_ForwardSearch, 0},
+	{"Menu", false, 9, cmd_ShowMenu, 0},
+	{"Quit", false, 10, cmd_Quit, 0},
+	{NULL, false, 0, 0, 0}
 };
 
 ST_MENU_ITEM _file[] = {
-	{"~S~ave", cmd_SaveData, "s"},
-	{"--"},
-	{"~R~aw output quit", cmd_RawOutputQuit, "M-q"},
-	{"E~x~it", cmd_Quit, "q, F10"},
-	{NULL}
+	{"~S~ave", cmd_SaveData, "s", 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"~R~aw output quit", cmd_RawOutputQuit, "M-q", 0, 0, 0, NULL},
+	{"E~x~it", cmd_Quit, "q, F10", 0, 0, 0, NULL},
+	{NULL, 0, NULL, 0, 0, 0, NULL}
 };
 
 /*
@@ -41,98 +50,100 @@ ST_MENU_ITEM _file[] = {
  * menu references on this content.
  */
 ST_MENU_ITEM _search[] = {
-	{"~S~earch", cmd_ForwardSearch, "/"},
-	{"Search ~b~ackward", cmd_BackwardSearch, "?"},
-	{"Search ~a~gain", cmd_SearchNext, "n"},
-	{"Search p~r~evious", cmd_SearchPrev, "N"},
-	{"--"},
-	{"Search ~c~olumn", cmd_SearchColumn, "c"},
-	{"--"},
-	{"~T~oggle bookmark", cmd_ToggleBookmark, "M-k"},
-	{"~P~rev bookmark", cmd_PrevBookmark, "M-i"},
-	{"~N~ext bookmark", cmd_NextBookmark, "M-j"},
-	{"~F~lush bookmarks", cmd_FlushBookmarks, "M-o"},
-	{NULL}
+	{"~S~earch", cmd_ForwardSearch, "/", 0, 0, 0, NULL},
+	{"Search ~b~ackward", cmd_BackwardSearch, "?", 0, 0, 0, NULL},
+	{"Search ~a~gain", cmd_SearchNext, "n", 0, 0, 0, NULL},
+	{"Search p~r~evious", cmd_SearchPrev, "N", 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"Search ~c~olumn", cmd_SearchColumn, "c", 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"~T~oggle bookmark", cmd_ToggleBookmark, "M-k", 0, 0, 0, NULL},
+	{"~P~rev bookmark", cmd_PrevBookmark, "M-i", 0, 0, 0, NULL},
+	{"~N~ext bookmark", cmd_NextBookmark, "M-j", 0, 0, 0, NULL},
+	{"~F~lush bookmarks", cmd_FlushBookmarks, "M-o", 0, 0, 0, NULL},
+	{NULL, 0, NULL, 0, 0, 0, NULL}
 };
 
 ST_MENU_ITEM _command[] = {
-	{"_0_Release fixed columns", cmd_ReleaseCols, "0"},
-	{"_1_Freeze one column", cmd_FreezeOneCol, "1"},
-	{"_2_Freeze two columns", cmd_FreezeTwoCols, "2"},
-	{"_3_Freeze three columns", cmd_FreezeThreeCols, "3"},
-	{"_4_Freeze four columns", cmd_FreezeFourCols, "4"},
-	{"--"},
-	{"~P~rev row", cmd_CursorUp, "k, Key up"},
-	{"~N~ext row", cmd_CursorDown, "j, Key down"},
-	{"Move to l~e~ft", cmd_MoveLeft, "h, Key left"},
-	{"Move to ~r~ight", cmd_MoveRight, "l, Key right"},
-	{"--"},
-	{"Go to ~f~irst row", cmd_CursorFirstRow, "g, C-Home"},
-	{"Go to l~a~st row", cmd_CursorLastRow, "G, C-End"},
-	{"Go to ~l~ine", cmd_GotoLine, "M-l"},
-	{"--"},
-	{"~S~how first column", cmd_ShowFirstCol, "^, Home"},
-	{"Sho~w~ last column", cmd_ShowLastCol, "$, End"},
-	{"--"},
-	{"Page up", cmd_PageUp, "C-b, Prev page"},
-	{"Page down", cmd_PageDown, "C-f, space, Next page"},
-	{"--"},
-	{"As~c~ending order", cmd_SortAsc, "a"},
-	{"~D~escending order", cmd_SortDesc, "d"},
-	{"~O~riginal order", cmd_OriginalSort, "u"},
-	{NULL}
+	{"_0_Release fixed columns", cmd_ReleaseCols, "0", 0, 0, 0, NULL},
+	{"_1_Freeze one column", cmd_FreezeOneCol, "1", 0, 0, 0, NULL},
+	{"_2_Freeze two columns", cmd_FreezeTwoCols, "2", 0, 0, 0, NULL},
+	{"_3_Freeze three columns", cmd_FreezeThreeCols, "3", 0, 0, 0, NULL},
+	{"_4_Freeze four columns", cmd_FreezeFourCols, "4", 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"~P~rev row", cmd_CursorUp, "k, Key up", 0, 0, 0, NULL},
+	{"~N~ext row", cmd_CursorDown, "j, Key down", 0, 0, 0, NULL},
+	{"Move to l~e~ft", cmd_MoveLeft, "h, Key left", 0, 0, 0, NULL},
+	{"Move to ~r~ight", cmd_MoveRight, "l, Key right", 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"Go to ~f~irst row", cmd_CursorFirstRow, "g, C-Home", 0, 0, 0, NULL},
+	{"Go to l~a~st row", cmd_CursorLastRow, "G, C-End", 0, 0, 0, NULL},
+	{"Go to ~l~ine", cmd_GotoLine, "M-l", 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"~S~how first column", cmd_ShowFirstCol, "^, Home", 0, 0, 0, NULL},
+	{"Sho~w~ last column", cmd_ShowLastCol, "$, End", 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"Page up", cmd_PageUp, "C-b, Prev page", 0, 0, 0, NULL},
+	{"Page down", cmd_PageDown, "C-f, space, Next page", 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"As~c~ending order", cmd_SortAsc, "a", 0, 0, 0, NULL},
+	{"~D~escending order", cmd_SortDesc, "d", 0, 0, 0, NULL},
+	{"~O~riginal order", cmd_OriginalSort, "u", 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"Refres~h~ screen", cmd_Refresh, "R, C-l", 0, 0, 0, NULL},
+	{NULL, 0, NULL, 0, 0, 0, NULL}
 };
 
 ST_MENU_ITEM _theme[] = {
-	{"_0_Midnight black", cmd_SetTheme_MidnightBlack},
-	{"_1_Midnight theme", cmd_SetTheme_Midnight},
-	{"_2_FoxPro like", cmd_SetTheme_Foxpro},
-	{"_3_Pdmenu like", cmd_SetTheme_Pdmenu},
-	{"_4_White theme", cmd_SetTheme_White},
-	{"_5_Mutt theme", cmd_SetTheme_Mutt},
-	{"_6_PC Fand like", cmd_SetTheme_Pcfand},
-	{"_7_Green theme", cmd_SetTheme_Green},
-	{"_8_Blue theme", cmd_SetTheme_Blue},
-	{"_9_Word perfect theme", cmd_SetTheme_WP},
-	{"_l_Low contrast blue theme", cmd_SetTheme_Lowcontrast},
-	{"_c_Dark cyan theme", cmd_SetTheme_Darkcyan},
-	{"_p_Paradox like", cmd_SetTheme_Paradox},
-	{"_d_DbaseIV retro", cmd_SetTheme_DBase},
-	{"_e_DbaseIV retro (Magenta)", cmd_SetTheme_DBasemagenta},
-	{"_r_Red white theme", cmd_SetTheme_Red},
-	{"_s_Simple theme", cmd_SetTheme_Simple},
-	{"_o_Solar Dark theme", cmd_SetTheme_SolarDark},
-	{"_g_Solar Light theme", cmd_SetTheme_SolarLight},
-	{"_u_Gruvbox Light theme", cmd_SetTheme_GruvboxLight},
-	{"_t_Tao Light theme", cmd_SetTheme_TaoLight},
+	{"_0_Midnight black", cmd_SetTheme_MidnightBlack, NULL, 0, 0, 0, NULL},
+	{"_1_Midnight theme", cmd_SetTheme_Midnight, NULL, 0, 0, 0, NULL},
+	{"_2_FoxPro like", cmd_SetTheme_Foxpro, NULL, 0, 0, 0, NULL},
+	{"_3_Pdmenu like", cmd_SetTheme_Pdmenu, NULL, 0, 0, 0, NULL},
+	{"_4_White theme", cmd_SetTheme_White, NULL, 0, 0, 0, NULL},
+	{"_5_Mutt theme", cmd_SetTheme_Mutt, NULL, 0, 0, 0, NULL},
+	{"_6_PC Fand like", cmd_SetTheme_Pcfand, NULL, 0, 0, 0, NULL},
+	{"_7_Green theme", cmd_SetTheme_Green, NULL, 0, 0, 0, NULL},
+	{"_8_Blue theme", cmd_SetTheme_Blue, NULL, 0, 0, 0, NULL},
+	{"_9_Word perfect theme", cmd_SetTheme_WP, NULL, 0, 0, 0, NULL},
+	{"_l_Low contrast blue theme", cmd_SetTheme_Lowcontrast, NULL, 0, 0, 0, NULL},
+	{"_c_Dark cyan theme", cmd_SetTheme_Darkcyan, NULL, 0, 0, 0, NULL},
+	{"_p_Paradox like", cmd_SetTheme_Paradox, NULL, 0, 0, 0, NULL},
+	{"_d_DbaseIV retro", cmd_SetTheme_DBase, NULL, 0, 0, 0, NULL},
+	{"_e_DbaseIV retro (Magenta)", cmd_SetTheme_DBasemagenta, NULL, 0, 0, 0, NULL},
+	{"_r_Red white theme", cmd_SetTheme_Red, NULL, 0, 0, 0, NULL},
+	{"_s_Simple theme", cmd_SetTheme_Simple, NULL, 0, 0, 0, NULL},
+	{"_o_Solar Dark theme", cmd_SetTheme_SolarDark, NULL, 0, 0, 0, NULL},
+	{"_g_Solar Light theme", cmd_SetTheme_SolarLight, NULL, 0, 0, 0, NULL},
+	{"_u_Gruvbox Light theme", cmd_SetTheme_GruvboxLight, NULL, 0, 0, 0, NULL},
+	{"_t_Tao Light theme", cmd_SetTheme_TaoLight, NULL, 0, 0, 0, NULL},
 	{NULL},
 };
 
 ST_MENU_ITEM _options[] = {
-	{"~C~ase sensitive search", cmd_CSSearchSet, NULL},
-	{"Case ~i~nsensitive search", cmd_CISearchSet, NULL},
-	{"~U~pper case sensitive search", cmd_USSearchSet, NULL},
-	{"--"},
-	{"Highlight searched ~l~ines", cmd_HighlightLines, NULL},
-	{"Highlight searched ~v~alues", cmd_HighlightValues, NULL},
-	{"~W~ithout highlighting", cmd_NoHighlight, NULL},
-	{"--"},
-	{"Show cursor", cmd_ShowCursor, "M-c"},
-	{"Show vertical cursor", cmd_ShowVerticalCursor, "M-v"},
-	{"Show line ~n~umbers", cmd_RowNumToggle, "M-n"},
-	{"Show top bar", cmd_ShowTopBar, NULL},
-	{"Show bottom bar", cmd_ShowBottomBar, NULL},
-	{"--"},
-	{"~M~ouse support", cmd_MouseToggle, "M-m"},
-	{"~Q~uiet mode", cmd_SoundToggle, NULL},
-	{"--"},
-	{"Force unicode ~b~orders", cmd_UtfArtToggle, NULL},
-	{"Force ~a~scii menu", cmd_MenuAsciiArtToggle, NULL},
-	{"Bold labels", cmd_BoldLabelsToggle, "M-b"},
-	{"Bold cursor", cmd_BoldCursorToggle, NULL},
+	{"~C~ase sensitive search", cmd_CSSearchSet, NULL, 0, 0, 0, NULL},
+	{"Case ~i~nsensitive search", cmd_CISearchSet, NULL, 0, 0, 0, NULL},
+	{"~U~pper case sensitive search", cmd_USSearchSet, NULL, 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"Highlight searched ~l~ines", cmd_HighlightLines, NULL, 0, 0, 0, NULL},
+	{"Highlight searched ~v~alues", cmd_HighlightValues, NULL, 0, 0, 0, NULL},
+	{"~W~ithout highlighting", cmd_NoHighlight, NULL, 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"Show cursor", cmd_ShowCursor, "M-c", 0, 0, 0, NULL},
+	{"Show vertical cursor", cmd_ShowVerticalCursor, "M-v", 0, 0, 0, NULL},
+	{"Show line ~n~umbers", cmd_RowNumToggle, "M-n", 0, 0, 0, NULL},
+	{"Show top bar", cmd_ShowTopBar, NULL, 0, 0, 0, NULL},
+	{"Show bottom bar", cmd_ShowBottomBar, NULL, 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"~M~ouse support", cmd_MouseToggle, "M-m", 0, 0, 0, NULL},
+	{"~Q~uiet mode", cmd_SoundToggle, NULL, 0, 0, 0, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"Force unicode ~b~orders", cmd_UtfArtToggle, NULL, 0, 0, 0, NULL},
+	{"Force ~a~scii menu", cmd_MenuAsciiArtToggle, NULL, 0, 0, 0, NULL},
+	{"Bold labels", cmd_BoldLabelsToggle, "M-b", 0, 0, 0, NULL},
+	{"Bold cursor", cmd_BoldCursorToggle, NULL, 0, 0, 0, NULL},
 	{"~T~heme", MENU_ITEM_THEME, NULL, 0, 0,  0, _theme},
-	{"--"},
-	{"~S~ave setup", cmd_SaveSetup, NULL},
+	{"--", 0, NULL, 0, 0, 0, NULL},
+	{"~S~ave setup", cmd_SaveSetup, NULL, 0, 0, 0, NULL},
 	{NULL},
 };
 
@@ -254,9 +265,10 @@ init_menu(struct ST_MENU *current_menu)
 	if (current_menu)
 	{
 		int		positions[1024];
+		int		*refvals[1024];
 
-		st_menu_save(current_menu, positions, 1023);
-		st_menu_load(menu, positions);
+		st_menu_save(current_menu, positions, refvals, 1023);
+		st_menu_load(menu, positions, refvals);
 		st_menu_free(current_menu);
 	}
 
@@ -264,11 +276,14 @@ init_menu(struct ST_MENU *current_menu)
 }
 
 struct ST_CMDBAR *
-init_cmdbar(struct ST_CMDBAR *current_cmdbar)
+init_cmdbar(struct ST_CMDBAR *current_cmdbar, Options *opts)
 {
 	struct ST_CMDBAR	   *cmdbar = NULL;
 
-	cmdbar = st_cmdbar_new(&menu_config, _bottombar);
+	if (opts->quit_on_f3)
+		cmdbar = st_cmdbar_new(&menu_config, _bottombar_alt1);
+	else
+		cmdbar = st_cmdbar_new(&menu_config, _bottombar);
 
 	if (current_cmdbar)
 	{
