@@ -3,7 +3,7 @@
  * print.c
  *	  visualisation loaded data
  *
- * Portions Copyright (c) 2017-2019 Pavel Stehule
+ * Portions Copyright (c) 2017-2020 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/print.c
@@ -54,7 +54,6 @@ flush_bytes(WINDOW *win,
 		    bool is_head_deco,
 		    bool is_bottom_deco,
 			DataDesc *desc,
-			ScrDesc *scrdesc,
 			Options *opts)
 {
 	if ((is_top_deco || is_head_deco || is_bottom_deco) &&
@@ -130,6 +129,215 @@ flush_bytes(WINDOW *win,
 	}
 }
 
+static void
+print_column_names(WINDOW *win,
+				   int srcx,						/* offset to displayed data */
+				   int vcursor_xmin,				/* xmin in display coordinates */
+				   int vcursor_xmax,				/* xmax in display coordinates */
+				   DataDesc *desc,
+				   Options *opts,
+				   Theme *t)
+{
+	char   *headline_ptr = desc->headline_transl;
+	char   *headline_end_ptr = headline_ptr + desc->headline_char_size;
+	char   *namesline = desc->namesline;
+	char   *ptr = namesline;
+	int		maxy, maxx;
+	int		cy, cx;
+	int		pos = 0;
+	int		bytes;
+	int		chars;
+	int		extra_space = desc->border_type == 0 ? 0 : 1;
+	attr_t		active_attr = 0;
+	attr_t		new_attr;
+	int		i;
+
+	getyx(win, cy, cx);
+	getmaxyx(win, maxy, maxx);
+
+	(void) cx;
+	(void) maxy;
+
+	/* skip left invisible chars */
+	while (pos < srcx)
+	{
+		bytes = utf8charlen(*ptr);
+		chars = utf_dsplen(ptr);
+
+		if (pos + chars > srcx)
+		{
+			wprintw(win, "%*s", pos + chars - srcx, "");
+
+			pos += chars;
+			ptr += bytes;
+			headline_ptr += chars;
+
+			break;
+		}
+
+		pos += chars;
+		ptr += bytes;
+		headline_ptr += chars;
+	}
+
+	/* position starts from zero again to be comparable with maxx */
+	pos -= srcx;
+
+	/* for each visible column (position) and defined colum */
+	while (pos < maxx && headline_ptr < headline_end_ptr)
+	{
+		char	column_format = *headline_ptr;
+		bool	is_cursor = vcursor_xmin <= pos && pos <= vcursor_xmax;
+
+		bytes = utf8charlen(*ptr);
+		chars = utf_dsplen(ptr);
+
+		if (is_cursor )
+			new_attr = column_format == 'd' ? t->cursor_data_attr : t->cursor_line_attr;
+		else
+			new_attr = column_format == 'd' ? t->data_attr : t->line_attr;
+
+		if (active_attr != new_attr)
+		{
+			/* disable current style */
+			wattroff(win, active_attr);
+
+			/* active new style */
+			active_attr = new_attr;
+			wattron(win, active_attr);
+		}
+
+		if (column_format != 'd')
+		{
+			if (desc->linestyle == 'a' && opts->force_uniborder)
+				waddch(win, ACS_VLINE);
+			else
+				wprintw(win, "%.*s", bytes, ptr);
+		}
+		else
+			wprintw(win, "%*s", chars, "");
+
+		headline_ptr += chars;
+		ptr += bytes;
+		pos += chars;
+	}
+
+	wattroff(win, active_attr);
+
+	/* check all column names and print that are visible */
+	for (i = 0; i < desc->columns; i++)
+	{
+		CRange *col = &desc->cranges[i];
+		int		reduced_xmin = col->xmin - srcx;
+		int		reduced_xmax = col->xmax - srcx;
+		int		visible_xmin;
+		int		visible_xmax;
+		int		visible_width;
+		int		x = -1;
+		int		skipbytes = 0;
+		char   *colname;
+		int		colname_size;
+		int		colname_width;
+
+		/* skip invisible columns */
+		if (reduced_xmax < 0)
+			continue;
+
+		/* leave when no other column can be visible */
+		if (reduced_xmin > maxx)
+			break;
+
+		colname = desc->namesline + col->name_offset;
+		colname_size = col->name_size;
+		colname_width = col->name_width;
+
+		visible_xmin = reduced_xmin > 1 ? reduced_xmin : 1;
+		visible_xmax = reduced_xmax < maxx ? reduced_xmax : maxx;
+		visible_width = visible_xmax - visible_xmin + extra_space;
+
+		if (reduced_xmin >= 1 && reduced_xmax <= maxx)
+		{
+			/* center content */
+			x = (visible_width - colname_width - 1) / 2 + visible_xmin + 1;
+		}
+		else if (reduced_xmin >= 1)
+		{
+			/*
+			 * display first visible chars from column name when all chars
+			 * from column name cannot be displayed, or center content.
+			 */
+			if (visible_width <= colname_width + 1 + extra_space)
+			{
+				char   *ptr = colname;
+				int		x2;
+
+				/* size of visible chars will be calculated */
+				colname_size = 0;
+
+				x = visible_xmin + 1 + extra_space;
+				x2 = x;
+
+				while (x2 <= maxx)
+				{
+					bytes = utf8charlen(*ptr);
+					chars = utf_dsplen(ptr);
+
+					if (x2 + chars > maxx)
+						break;
+
+					colname_size += bytes;
+					x2 += chars;
+					ptr += bytes;
+				}
+			}
+			else
+				x = (visible_width - colname_width - 1) / 2 + visible_xmin + 1;
+		}
+		else if (reduced_xmin < 1)
+		{
+			/* display last visible chars or center content */
+			if (visible_width <= col->name_width + extra_space)
+			{
+				char   *ptr = colname;
+				int		reduce_width = colname_width + extra_space - visible_width;
+
+				/* there is not a space from left */
+				x = visible_xmin - 1;
+
+				if (reduce_width < colname_width)
+				{
+					while (reduce_width > 0)
+					{
+						bytes = utf8charlen(*ptr);
+						chars = utf_dsplen(ptr);
+
+						skipbytes += bytes;
+						reduce_width -= chars;
+						ptr += bytes;
+					}
+				}
+				else
+					/* there is not any content to print */
+					x = -1;
+
+				if (reduce_width < 0 && x != -1)
+					x += abs(reduce_width);
+			}
+			else
+				x = (visible_width - col->name_width - 1) / 2 + visible_xmin;
+		}
+
+		new_attr = (vcursor_xmin <= x && x <= vcursor_xmax) ? t->cursor_data_attr : t->data_attr;
+
+		if (x != -1)
+		{
+			wattron(win, new_attr);
+			mvwprintw(win, cy, x, "%.*s", colname_size - skipbytes, colname + skipbytes);
+			wattroff(win, new_attr);
+		}
+	}
+}
+
 void
 window_fill(int window_identifier,
 			int srcy,
@@ -155,6 +363,7 @@ window_fill(int window_identifier,
 	bool		is_fix_rows = window_identifier == WINDOW_LUC || window_identifier == WINDOW_FIX_ROWS;
 	bool		is_rownum = window_identifier == WINDOW_ROWNUM;
 	bool		is_rownum_luc = window_identifier == WINDOW_ROWNUM_LUC;
+	bool		is_fix_rows_only = window_identifier == WINDOW_FIX_ROWS;
 
 	win = scrdesc->wins[window_identifier];
 	t = &scrdesc->themes[window_identifier];
@@ -265,7 +474,7 @@ window_fill(int window_identifier,
 
 				lnb->lineinfo = malloc(1000 * sizeof(LineInfo));
 				if (lnb->lineinfo == NULL)
-					leave_ncurses("out of memory");
+					leave("out of memory");
 
 				memset(lnb->lineinfo, 0, 1000 * sizeof(LineInfo));
 
@@ -345,7 +554,7 @@ window_fill(int window_identifier,
 
 				if (str != NULL)
 				{
-					positions[npositions][0] = opts->force8bit ? str - rowstr : utf8len_start_stop(rowstr, str);
+					positions[npositions][0] = opts->force8bit ? (size_t) (str - rowstr) : utf8len_start_stop(rowstr, str);
 					positions[npositions][1] = positions[npositions][0] + scrdesc->searchterm_char_size;
 
 					/* don't search more if we are over visible part */
@@ -461,6 +670,16 @@ window_fill(int window_identifier,
 				is_expand_head = false;
 			}
 
+			/*
+			 * To ensure visible column names how it can be possible, we should to
+			 * print column names directly (not like input document row.
+			 */
+			if (is_fix_rows_only && rowstr == desc->namesline )
+			{
+				print_column_names(win, srcx, vcursor_xmin, vcursor_xmax, desc, opts, t);
+				continue;
+			}
+
 			/* skip first srcx chars */
 			i = srcx;
 			left_spaces = 0;
@@ -501,9 +720,7 @@ window_fill(int window_identifier,
 				char   *p;
 				int		aux_left_spaces = left_spaces;
 
-				free_row = malloc(left_spaces + strlen(rowstr) + 1);
-				if (!free_row)
-					leave_ncurses("out of memory");
+				free_row = smalloc(left_spaces + strlen(rowstr) + 1);
 
 				p = free_row;
 				while (aux_left_spaces-- > 0)
@@ -712,7 +929,7 @@ window_fill(int window_identifier,
 					{
 						if (!is_footer)
 						{
-							int		new_attr;
+							attr_t		new_attr;
 
 							if (is_cross_cursor)
 							{
@@ -739,7 +956,6 @@ window_fill(int window_identifier,
 												is_head_deco,
 												is_bottom_deco,
 												desc,
-												scrdesc,
 					  							opts);
 
 
@@ -809,7 +1025,6 @@ window_fill(int window_identifier,
 							is_head_deco,
 							is_bottom_deco,
 							desc,
-							scrdesc,
 							opts);
 
 			/* print trailing spaces necessary for correct vertical cursor */
@@ -873,6 +1088,8 @@ ansi_attr(attr_t attr)
 
 #ifndef COLORIZED_NO_ALTERNATE_SCREEN
 
+	(void) attr;
+
 	return "";
 
 #else
@@ -886,10 +1103,10 @@ ansi_attr(attr_t attr)
 
 	if ((attr & A_BOLD) != 0)
 	{
-		snprintf(result, 20, "\e[1;%d;%dm", fc, bc);
+		snprintf(result, 20, "\033[1;%d;%dm", fc, bc);
 	}
 	else
-		snprintf(result, 20, "\e[0;%d;%dm", fc, bc);
+		snprintf(result, 20, "\033[0;%d;%dm", fc, bc);
 
 	return result;
 
@@ -928,7 +1145,8 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 	row = 0;
 
 	if (offsety)
-		printf("\e[%dB", offsety);
+		/* \033 is \e */
+		printf("\033[%dB", offsety);
 
 	while (row < maxy)
 	{
@@ -993,7 +1211,7 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 			}
 
 			if (offsetx != 0)
-				printf("\e[%dC", offsetx);
+				printf("\033[%dC", offsetx);
 
 			/* skip first srcx chars */
 			i = srcx;
@@ -1124,7 +1342,7 @@ draw_rectange(int offsety, int offsetx,			/* y, x offset on screen */
 			{
 				printf("%.*s", bytes, rowstr);
 				if (clreoln)
-					printf("\e[K");
+					printf("\033[K");
 				printf("\n");
 			}
 
@@ -1153,10 +1371,10 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 										desc->last_row + 1);
 
 		for (i = 0; i < expected_rows; i++)
-			printf("\eD");
+			printf("\033D");
 
 		/* Go wit cursor to up */
-		printf("\e[%dA", expected_rows);
+		printf("\033[%dA", expected_rows);
 
 		/* now, the screen can be different, due prompt */
 		scrdesc->rows_rows = min_int(scrdesc->rows_rows,
@@ -1165,7 +1383,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 		/*
 		 * Save cursor - Attention, there are a Fedora29 bug, and it doesn't work
 		 */
-		printf("\e7");
+		printf("\0337");
 
 		if (scrdesc->fix_cols_cols > 0)
 		{
@@ -1180,7 +1398,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 		if (scrdesc->fix_rows_rows > 0 )
 		{
 			/* Go to saved position */
-			printf("\e8\e7");
+			printf("\0338\0337");
 
 			draw_rectange(0, scrdesc->fix_cols_cols,
 						  scrdesc->fix_rows_rows, size.ws_col - scrdesc->fix_cols_cols,
@@ -1193,7 +1411,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 		if (scrdesc->fix_rows_rows > 0 && scrdesc->fix_cols_cols > 0)
 		{
 			/* Go to saved position */
-			printf("\e8\e7");
+			printf("\0338\0337");
 
 			draw_rectange(0, 0,
 						  scrdesc->fix_rows_rows, scrdesc->fix_cols_cols,
@@ -1206,7 +1424,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 		if (scrdesc->rows_rows > 0)
 		{
 			/* Go to saved position */
-			printf("\e8\e7");
+			printf("\0338\0337");
 
 			draw_rectange(scrdesc->fix_rows_rows, scrdesc->fix_cols_cols,
 						  scrdesc->rows_rows, size.ws_col - scrdesc->fix_cols_cols,
@@ -1221,7 +1439,7 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 		if (w_footer(scrdesc) != NULL)
 		{
 			/* Go to saved position */
-			printf("\e8\e7");
+			printf("\0338\0337");
 
 			draw_rectange(scrdesc->fix_rows_rows + scrdesc->rows_rows, 0,
 						  scrdesc->footer_rows, scrdesc->maxx,
@@ -1231,6 +1449,6 @@ draw_data(Options *opts, ScrDesc *scrdesc, DataDesc *desc,
 		}
 
 		/* reset */
-		printf("\e[0m\r");
+		printf("\033[0m\r");
 	}
 }
